import json
from datetime import date, timedelta
from ..db import db, TaskGenerator, GeneratorType
from .tasks import add_task


def get_generator(item_id: int) -> TaskGenerator:
    return TaskGenerator.get_by_id(item_id)


def get_generators() -> list[TaskGenerator]:
    query = TaskGenerator.select().where(~TaskGenerator.deleted)
    return query.order_by("type", "template")


def add_generator(
    template: str,
    type: GeneratorType,
    val: str,
) -> TaskGenerator:
    # JSON for future expansion
    config = json.dumps({"val": val})
    with db.atomic():
        task = TaskGenerator.create(
            template=template,
            type=type,
            config=config,
        )
    return task


def generate_needed_tasks():
    to_create = []
    for g in get_generators():
        next = g.next_at()
        if not next:
            continue
        # TODO: make configurable
        if date.today() - next > timedelta(days=14):
            to_create.append(
                {
                    "text": g.template.format(next=next),
                    "category": "recurring",
                    "due": next,
                }
            )

    for c in to_create:
        add_task(**c)

    return to_create


def update_generator(
    item_id: int,
    **kwargs,
) -> TaskGenerator:
    # replace "val" with JSON
    if "val" in kwargs:
        config = {"val": kwargs.pop("val")}
        kwargs["config"] = json.dumps(config)
    with db.atomic():
        query = TaskGenerator.update(kwargs).where(TaskGenerator.id == item_id)
        query.execute()
        task = TaskGenerator.get_by_id(item_id)
    return task
